-- Title: Skill Helper
-- Author: JerichoHM
-- Version: 2.1a
-- Desc: A simple addon for tracking and using skills

-- GLOBALS
local SKILLCAP = 525
local y = -5
local imgFolder = "Interface\\AddOns\\SkillHelper\\IMAGES\\"
local shFrameBG = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background.blp", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 32, edgeSize = 16, insets = {left = 3, right = 3, top = 3, bottom = 3}}
local shBarBG = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", edgeFile = nil, tile = false, tileSize = 32, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}}
SLASH_SKILLHELPER1 = '/SH'

-- Frame Definitions
local shFrame

-- Skills Table
local skills = {}

-- Frames Table
local bars = {}

function shMain()
	shMainFrame()
end

function shMainFrame()
	-- Create Core UI Frame
	shFrame = CreateFrame("Frame", "shFrame", UIParent)
	shFrame:SetFrameStrata("DIALOG")
	shFrame:SetPoint("CENTER")
	shFrame:SetWidth(170)
	shFrame:SetHeight(28)
	shFrame:SetBackdrop(shFrameBG)
	
	--Mouse Enabling, Movement constraints, and Movment functions
	shFrame:SetClampedToScreen(true)
	if lock == nil then
		lock = true
		shFrame:EnableMouse(true)
		shFrame:SetMovable(true)
	else
		shFrame:EnableMouse(lock)
		shFrame:SetMovable(lock)
	end
	shFrame:RegisterForDrag("LeftButton")
	shFrame:SetScript("OnDragStart", shFrame.StartMoving)
	shFrame:SetScript("OnDragStop", shFrame.StopMovingOrSizing)
	
	-- Event Handlers
	shFrame:RegisterEvent("SKILL_LINES_CHANGED")
	shFrame:RegisterEvent("ADDON_LOADED")
	shFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	shFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	shFrame:SetScript("OnEvent", shOnEvent)
	
	shFrame:Show()
end

function shDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
	local bar = bars[name]
	if not bar then
		bar = CreateFrame("StatusBar", nil, shFrame, nil)
		bar:SetFrameStrata("DIALOG")
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetSize(160,18)
		bar:SetBackdrop(shBarBG)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetStatusBarTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar")
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetMinMaxValues(0,1)
		bar:SetValue((rank + rankModifier)/maxRank)
		
		local barTxt = bar:CreateFontString(nil, "ARTWORK")
		isValid = barTxt:SetFont("Fonts\\FRIZQT__.TTF", 10)
		barTxt:SetPoint("CENTER", bar, "CENTER", 0, 1)
		bar.text = barTxt
		
		local barBtn1 = CreateFrame("Button", nil, bar, "SecureActionButtonTemplate")
		barBtn1:SetFrameStrata("DIALOG")
		barBtn1:SetPoint("TOP", bar, "TOPRIGHT", 13, 0)
		barBtn1:SetSize(18,18)
		barBtn1:EnableMouse(true)
		barBtn1:SetAttribute("type", "spell");
		barBtn1:SetAttribute("spell", shGetSpell(name, 1));
		barBtn1BG = { bgFile = imgFolder .. shGetButton(name, 1) .. ".BLP", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
		barBtn1:SetBackdrop(barBtn1BG)
		bar.btn1 = barBtn1
		
		if numSpells == 2 then
			if name ~= "Skinning" and name ~= "Tailoring" then
				local barBtn2 = CreateFrame("Button", nil, barBtn1, "SecureActionButtonTemplate")
				barBtn2:SetFrameStrata("DIALOG")
				barBtn2:SetPoint("TOP", barBtn1, "TOPRIGHT", 10, 0)
				barBtn2:SetSize(18,18)
				barBtn2:EnableMouse(true)
				barBtn2:SetAttribute("type", "spell");
				barBtn2:SetAttribute("spell", shGetSpell(name, 2));
				barBtn2BG = { bgFile = imgFolder .. shGetButton(name, 2) .. ".BLP", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
				barBtn2:SetBackdrop(barBtn2BG)
				bar.btn2 = barBtn2
			end
		end
		
		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
		
		bars[name] = bar
	else
		bar:ClearAllPoints()
	end
	
	if	rank >= maxRank - 25 and maxRank < SKILLCAP then
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetStatusBarColor(0.7, 0.0, 0.0, 1.0)
		bar:SetBackdropColor(0.7, 0.0, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)
	else
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)
	end
	
	shFrame:SetHeight((y - 23)* -1)
end

function shUpdateData()
	-- Capture the SkillIndes for the players Professions
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
	
	-- Build an array of the Professions
	local profs = {prof1, prof2, archaeology, fishing, cooking, firstAid}
	
	for k, v in pairs(profs) do 
		-- Fetch the details for each Profession 
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier = GetProfessionInfo(v)
		shDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
		y = y - 18
		
		table.insert( skills, name)
	end
	
	shCleanUp()
	
	y = -5
end

function shCleanUp()
	local match = false
	local clnBar
	
	for k, d in pairs(bars) do
		for i, v in ipairs(skills) do
			if k == v then
				match = true
				break
			else
				match = false
			end
		end
		
		clnBar = bars[k]
		
		if match == false then
			clnBar:Hide()
			match = false
		elseif match == true then
			clnBar:Show()
		end
	end
	
	while ((# skills) > 0) do
		table.remove( skills, 1 )
	end
end

function shOnEvent(self, event)
	if event == "SKILL_LINES_CHANGED" then
		shUpdateData()
	elseif event == "ADDON_LOADED" then
		if autohide == nil then
			autohide = false
		end
		if lock == nil then
			lock = false
		end
	elseif event == "PLAYER_REGEN_DISABLED" then
		if autohide == true then
			shFrame:Hide()
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		if autohide == true then
			shFrame:Show()
		end
	end
end

function shGetButton(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Campfire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "FA"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Miling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospect"
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Smelt"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

function shGetSpell(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Basic Campfire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "First Aid"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return "Herb Gathering"
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospect"
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Smelting"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "MoA"
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

function SlashCmdList.SKILLHELPER(msg)
	if msg == "show" then
		if shFrame:IsVisible() == 1 then
			shFrame:Hide()
			print("Skill Helper hidden")
		else
			shFrame:Show()
			print("Skill Helper visible")
		end
	elseif msg == "autohide" then
		if autohide == true then
			autohide = false
			print("Autohide while in combat set to: OFF")
		elseif autohide == false then
			autohide = true
			print("Autohide while in combat set to: ON")
		end
	elseif msg == "lock" then
		if lock == true then
			lock = false
			shFrame:EnableMouse(lock)
			shFrame:SetMovable(lock)
			print("Skill Helper locked")
		elseif lock == false then
			lock = true
			shFrame:EnableMouse(lock)
			shFrame:SetMovable(lock)
			print("Skill Helper unlocked")
		end
	elseif msg == "reset" then
		shFrame:SetPoint("CENTER", UIParent, "CENTER")
		print("Skill Helper Reset")
	elseif msg == "settings" then
		if shFrame:IsVisible() == 1 then
			print("Frame Visible: True")
		else
			print("Frame Visible: False")
		end
		if autohide == true then
			print("Autohide Enabled: True")
		else
			print("Autohide Enabled: False")
		end
		if lock == true then
			print("Lock Enabled: False")
		else
			print("Lock Enabled: True")
		end
	else
		print("Skill Helper Slash Commands")
		print("type /SH show to visibility")
		print("type /SH autohide to toggle autohide while in combat")
		print("type /SH lock to toggle locking")
		print("type /SH reset to reset to default position")
		print("tpye /SH settigns to view current settings")
	end
end

shMain()